-----------------------------------------------------------------------------
-- This implements "channel variables" exactly as described in the paper
--
--   "Concurrent Haskell"
--   Simon Peyton Jones, Andrew Gordon and Sigbjorn Finne.
--   In Proceedings of the ACM Symposium on Principles of Programming
--   Languages,St Petersburg Beach, Florida, January 1996. 
--   http://www.dcs.gla.ac.uk/fp/authors/Simon_Peyton_Jones/
--     concurrent-haskell.ps
--
-----------------------------------------------------------------------------

module CVar(
       CVar(..),
       newCVar, putCVar, getCVar
       ) where

import MVar(
	MVar, newMVar, takeMVar, putMVar
	)

type CVar a = (MVar a,   -- Producer -> consumer
               MVar ())  -- Consumer -> producer

newCVar :: IO (CVar a)
putCVar :: CVar a -> a -> IO ()
getCVar :: CVar a -> IO a

newCVar
  = newMVar                 >>= \ data_var ->
    newMVar                 >>= \ ack_var ->
    putMVar ack_var ()      >>
    return (data_var, ack_var)

putCVar (data_var, ack_var) val
  = takeMVar ack_var >>
    putMVar data_var val

getCVar (data_var, ack_var)
  = takeMVar data_var      >>= \ val ->
    putMVar ack_var ()     >>
    return val


